/* 
    Automated Drink Mixer

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

    (C) 2007 Jason Hunt
    nulluser@gmail.com
    
    File: input.cpp
*/

#include "core.h"

/* Deal with mouse button press */
void input_button_press(unsigned int x, unsigned int y)
{
    unsigned int button = y / (display_y / 5);
    
    if (display_mode == DRINK_LIST)         // Drink list mode
    {
        if (button == 0) input_list_pageup();
        if (button == 1) input_list_up();
        if (button == 2) display_set_mode(DRINK_INFO);
        if (button == 3) input_list_down();
        if (button == 4) input_list_pagedown();
    } else
    if (display_mode == DRINK_INFO)         // Drink info mode
    {
        // Halt button pressed
        if (button == 2 && mixer_making_drink) mixer_halt_drink(); else

        // lock other buttons when making drink
        if (!mixer_making_drink)
        {
            if (button == 0) display_set_mode(DRINK_LIST);
            if (button == 1) input_list_up();
            if (button == 3) input_list_down();
            if (button == 4) mixer_make_cur_drink();
        }
    } 
}
/* End of button press */


/* Mouse was pressed over a drink */
void input_select_drink(unsigned int y_index)
{
    if (y_index > LIST_DISPLAY_SIZE -1 )
        y_index = LIST_DISPLAY_SIZE - 1;
                   
    sel_drink = sel_base + y_index;
    
    if (sel_drink > num_drinks)
        sel_drink = 0;
    
    display_set_mode(DRINK_INFO);
}
/* End of select_drink */


/* Scroll list up */
void input_list_up( void )
{
    if (sel_drink > 0) sel_drink--;
    if (sel_drink < sel_base) sel_base--;
}
/* End of list_up */


/* Scroll list down */
void input_list_down( void )
{
    if (sel_drink < num_drinks - 1) sel_drink++;
    if (sel_drink > sel_base + LIST_DISPLAY_SIZE - 1) sel_base++;
}
/* End of list_down */


/* Pageup on list */
void input_list_pageup( void )
{
    sel_drink -= LIST_DISPLAY_SIZE;

   if (sel_drink < 0) sel_drink = 0;

   if (sel_drink < sel_base)
       sel_base = sel_drink;
}
/* End of list_pageup */


/* Page down on list */
void input_list_pagedown( void )
{
    sel_drink += LIST_DISPLAY_SIZE;

    if (sel_drink > num_drinks - 1)
        sel_drink = num_drinks - 1;

   if (sel_drink > sel_base + LIST_DISPLAY_SIZE -1)
        sel_base = sel_drink;

    if (sel_base +  LIST_DISPLAY_SIZE > num_drinks)
    {
        sel_base = num_drinks -  LIST_DISPLAY_SIZE;
        sel_drink = sel_base;
    }
}
/* End of list_pagedown */


