/* 
    Automated Drink Mixer

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

    (C) 2007 Jason Hunt
    nulluser@gmail.com
    
    File: mixer.cpp
*/

#include "core.h"


int mixer_open_channels = 0; // CUrrent number of open mixing channels

bool mixer_making_drink = false; // True if currently making a drink

channel_stat_type *mixer_channel_stats = NULL; // Time stats for channels


/* Make sure all channels are closed */
void mixer_close_all( void )
{
    for (int c = 0; c < num_channels; c++)
    {
        char tmp[40];
        sprintf(tmp, "CLosing channel %d", c);
        debug("mixer_close_all", tmp);

        unsigned char command = c << 1; // Create command to close channel
    //    send_serial(command);     
    }
    
    debug("mixer_close_all", "done");
}
/* End of mixer_close_all */



/* Stop a drink in progress */
void mixer_halt_drink( void )
{
  //  PlaySound("..\\wave\\halt.wav", 0, SND_FILENAME | SND_ASYNC);    
    
    for (int i = 0; i < 12; i++)
    {
        SetTimer(main_hwnd, i+1024, 0, NULL); // Fire timer for channel
        KillTimer(main_hwnd, i+2048);         // Halt update timers
    }

    mixer_open_channels = 0;          // No channels open
    mixer_making_drink = false;       // DOne making the drink
    
    InvalidateRect(main_hwnd, NULL, 0); // Force display update
}
/* End of halt_drink */


/* Close a mixing channel when done with ingredient */
void mixer_close_channel(unsigned int c)
{              
    char tmp[400];
    sprintf(tmp, "Closing channel %d at %f\n", c, get_time());
    debug("close_channel", tmp);

    KillTimer(main_hwnd, c+1024);   // Done using this timer
    
    mixer_open_channels--;                // One less channel open now

    unsigned char command = c << 1; // Create command to close channel
//    send_serial(command);                
               
    // See if we are done making the drink
    if (mixer_open_channels == 0)
    {
        debug("close_channel", "Done making drink\n");
        mixer_making_drink = false;

        // Stop update timer
        KillTimer(main_hwnd, UPDATE_TIMER);
 
        InvalidateRect(main_hwnd, NULL, 0);
    }
}
/* End of close_channel */


/* Open a channel for mixing */
void mixer_open_channel(unsigned int c, unsigned int duration)
{
    char tmp[400];
    sprintf(tmp, "Opening channel %d at %f\n", c, get_time());
    debug("open_channel", tmp);

    // Create duration timer
    SetTimer(main_hwnd, c+1024, (int)duration, NULL);

    // Set channels time stats
    mixer_channel_stats[c].start_time = get_time();
    mixer_channel_stats[c].duration = duration/1000.0;
    mixer_channel_stats[c].complete = 0;

    // One more channel is now open
    mixer_open_channels++;

    unsigned char command = ((c) << 1) | 1;// Create command to open channel
//    send_serial(command);*/
}
/* End of open_channel */


/* Update progress bar for channel */
void mixer_update_channels( void )
{
    double t = get_time();

    for (unsigned int i = 0; i < num_channels; i++) 
        if (mixer_channel_stats[i].start_time > 0)
        {
    
            // Compute compleation indicator
            mixer_channel_stats[i].complete = (t - mixer_channel_stats[i].start_time) / 
                                        mixer_channel_stats[i].duration;

            // Clamp complete
            if (mixer_channel_stats[i].complete < 0)
                mixer_channel_stats[i].complete = 0;
    
            if (mixer_channel_stats[i].complete >= 1)
                mixer_channel_stats[i].complete  = 1;
            
        }    
    
    // Force screen update
    InvalidateRect(main_hwnd, NULL, 0);
}
/* End of update channel */


/* Make a drink */
void mixer_make_drink(unsigned int drink_num)
{
    char tmp[400];
      
    if (num_drinks == 0) return;
      
    if (drinks[drink_num].num_mixers == 0) return; // Nothing to mix

    //clear all channel start times
    for (int i = 0; i < num_channels; i++)
        mixer_channel_stats[i].start_time = 0;

    mixer_open_channels = 0;
    
    sprintf(tmp, "Making a %s\n", drinks[drink_num].name);
    debug("make_drink", tmp);

    // Add all of the mixers
    for (int i = 0; i < drinks[drink_num].num_mixers; i++)
    {
        float volume = drinks[drink_num].mixers[i].volume;
        
        // COrrect for flow rate
        volume *= drinks[drink_num].mixers[i].pour_rate;
        
        unsigned int  channel = drinks[drink_num].mixers[i].channel;
                
        // Add the mixer if the channel was found and the amount is non zero
        if (channel != -1 && volume > 0)                   
        {
            mixer_open_channel(channel-1, (int)(volume * 1000));  
            mixer_making_drink = true;
        }

    }
    
    // Create the progress timer if we are actually making a drink
    if (mixer_making_drink)
    {
        sound_play_random();
        SetTimer(main_hwnd, UPDATE_TIMER, 10, NULL);
    }


}
/* End of make drink */


/* Make the selected drink */
void mixer_make_cur_drink(void)
{
    mixer_make_drink(sel_drink);
}
/* End of make_cur_drink */



