/* 
    Automated Drink Mixer

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

    (C) 2007 Jason Hunt
    nulluser@gmail.com
    
    File: types.cpp
*/


#ifndef TYPES_H
#define TYPES_H


#define INACTIVE_TIMEOUT   60

// True if compiled for touch screen
#define TOUCHSCREEN_MODE    0

// Mysql database connection info
#define MYSQL_HOST   "127.0.0.1"
#define MYSQL_USER   "localbar"
#define MYSQL_PASS   "localbar"
#define MYSQL_DBASE  "bar"

// Display modes
#define DRINK_LIST  0
#define DRINK_INFO  1

// Hard coded constants
#define NUM_MODES                  2  // Number of display modes
#define NUM_BUTTONS                5  // Number of command buttons

#define CHANNEL_TIMER           1024  // Base for channel timers
#define UPDATE_TIMER            2048  // Timer ID for update

// Parameters
#define SOUND_PATH   "d:\\bar\\wave"  // Path to load sound files from
#define LOG_FILE_NAME      "log.txt"  // Location for log file
#define ARROW_WIDTH_RATIO      0.008  // Width of arrow in relation to display x size
#define ARROW_LENGTH_RATIO     0.025  // Height of arrow in relation to display x size
#define LIST_DISPLAY_SIZE         12  // Number of items in drink list
#define CHARS_PER_LINE            30  // Number of characters in each line of list text
#define PROGRESS_BAR_WIDTH       0.5  // Width of progress bar wrt display x

// Different types of command buttons
#define BUTTON_NONE     0
#define BUTTON_UP       1
#define BUTTON_PGUP     2
#define BUTTON_DOWN     3
#define BUTTON_PGDOWN   4
#define BUTTON_BACK     5
#define BUTTON_GO       6
#define BUTTON_HALT     7

// Arrow defs
#define ARROW_UP        0
#define ARROW_DOWN      1
#define ARROW_LEFT      2
#define ARROW_RIGHT     3

// Channel not found indicator
#define NO_CHANNEL     -1

// Time stats for channel
typedef struct 
{
    float start_time;       // Time channel was opened
    float duration;         // How long the channel needs to be on
    float complete;         // Completation percentage
           
} channel_stat_type;


// Data for a mixer
typedef struct
{
    char *name;                 // Ingredient name
    int channel;                // Channel the ingredient is loaded into
    float volume;               // Amount of pour
    float cost;                 // Cost per ounce
    float content;              // Alchol content
    float pour_rate;            // Pour rate of drink
} mixer_type;


// Data for a drink
typedef struct
{
    char *name;                 // Name of drink
    mixer_type *mixers;         // Structure of mixers
    unsigned int num_mixers;    // Number of mixers
    bool mixers_found;          // True if all mixers are avaiable
} drink_type;

#endif

