/* 
    Automated Drink Mixer

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

    (C) 2007 Jason Hunt
    nulluser@gmail.com
    
    File: config.cpp
*/


#include "core.h"

// Parameters

char log_file_name[STR];            // Filename for log
char serial_port[STR];
float arrow_width_ratio;            // Nav arrow width
float arrow_length_ratio;           // Nav arrow length

char mysql_host[STR];	// Location my mysql server
char mysql_user[STR];	// User
char mysql_pass[STR];	// Pass
char mysql_dbase[STR];	// Database name


unsigned int drink_list_size;       // Size of drink list display
unsigned int chars_per_line;        // Number of char per line on display
float progress_bar_width;           // Size of progress bar
int fullscreen;                     // 1 if in full screen mode
int inactive_timeout;               // Inactive seconds before database reload
int touchscreen_mode;               // 1 if in touchscreen mode
int trigger;                        // Trigger disr for cup sensor





COLORREF menu_color;                // Color for normal GUI objects
COLORREF na_color;                  // Color for objects not avaiable
COLORREF button_color;              // Color for nac buttons
COLORREF outline_color;


/* Set default values */
void config_defaults( void )
{
    strcpy(log_file_name, "log.txt");
    strcpy(serial_port, "COM1");
    
    strcpy(list_image.filename, "");
    strcpy(info_image.filename, "");
       
    strcpy(mysql_host, "");
    strcpy(mysql_user, "");
    strcpy(mysql_pass, "");
    strcpy(mysql_dbase, "");
        
    display_x = 1024; 
    display_y = 768;

    arrow_width_ratio = 0.008;
    arrow_length_ratio = 0.025;
    drink_list_size  = 12;
    chars_per_line  = 30;
    progress_bar_width = 0.5;
        
    fullscreen = 0;    
    touchscreen_mode = 0;
    
    inactive_timeout = 60;
    
    trigger = 128;
    
    
    menu_color = RGB(255, 255, 255);    // Color for normal GUI objects
    na_color = RGB(255, 0, 0);      // Color for objects not avaiable
    button_color = RGB(255, 255, 255);
    outline_color = RGB(128,128, 255);    
}
/* End of config_defaults */


/* Gets the next string in a line, up to a space or endline */
void get_next_string(char *line, unsigned int &i, char *str)
{
    int done_str = 0;   // Flag for done with string
    
    unsigned int s = 0; // Index into target string

    // Eat white space at front of string
    while ((line[i] == ' ' || line[i] == '\t') && i < STR) i++;
  
    while (!done_str && i < STR)
    {
        char ch = line[i++];
                               
        if (ch == ' ' || ch == '\t' ||  ch == 0 || ch == '\n')
            done_str = 1;
        else
            str[s++] = ch;
    }

    str[s] = 0;   // null terminate
}
/* End of get_next_string */


/*  Get RGB color from a string */
COLORREF get_color(char *str)
{
    unsigned int r, g, b;

    sscanf(str, "%2x%2x%2x", &r, &g, &b);
    
    return(RGB(r, g, b));
}
/* End of get_color */


/* Parse a config line for commands */
void parse_line(char *line)
{
    unsigned int index = 0;
    char str[STR], data[STR];
    
    get_next_string(line, index, str); // Get command 
    get_next_string(line, index, data); // Get data 

    if (!stricmp("display_x", str))                   
    {        
        debug("Set display_x", data);       
        display_x = atoi(data);
    } 

    if (!stricmp("display_y", str))            
    {
        debug("Set display_y", data);       
        display_y = atoi(data);
    } 

    if (!stricmp("fullscreen", str))               
    { 
        debug("Set fullscreen", data);        
        fullscreen = atoi(data);
    } 
    
    if (!stricmp("list_image", str))               
    { 
        debug("Set background", data);        
        strcpy(list_image.filename, data);
    }     
    
    if (!stricmp("info_image", str))               
    { 
        debug("Set background", data);        
        strcpy(info_image.filename, data);
    }       
    
    if (!stricmp("log_file_name", str))                
    {   
        debug("Set log_file_name", data);     
        strcpy(log_file_name, data);
    } 

    if (!stricmp("serial_port", str))                
    {   
        debug("Set serial_port", data);     
        strcpy(serial_port, data);
    } 
    
    if (!stricmp("trigger", str))                   
    {        
        debug("Set trigger", data);       
        trigger = atoi(data);
    }     
    
    if (!stricmp("load_sounds", str))                  
    {    
        debug("load_sounds", data);
        sound_load_path(data);
    } 

    if (!stricmp("arrow_width_ratio", str))                
    { 
        debug("Set arrow_width_ratio", data);
        arrow_width_ratio = atof(data);
    } 
    
    if (!stricmp("arrow_length_ratio", str))            
    {       
        debug("Set arrow_length_ratio", data);        
        arrow_length_ratio = atof(data);
    } 
    
    if (!stricmp("drink_list_size", str))                 
    {
        debug("Set drink_list_size", data);      
        drink_list_size = atoi(data);
    } 
    
    if (!stricmp("chars_per_line", str))            
    {
        debug("Set chars_per_line", data);
        chars_per_line = atoi(data);
    } 
    
    if (!stricmp("progress_bar_width", str))                  
    {
        debug("Set progress_bar_width", data);
        progress_bar_width = atof(data);
    } 
    
    if (!stricmp("inactive_timeout", str))                  
    {
        debug("Set inactive_timeout", data);
        inactive_timeout = atoi(data);
    } 
    
    if (!stricmp("touchscreen_mode", str))                  
    {
        debug("Set touchscreen_mode", data);
        touchscreen_mode = atoi(data);
    }     
    
    if (!stricmp("menu_color", str))                  
    {
        debug("Set menucolor", data);
        menu_color = get_color(data);
    }    
     
    if (!stricmp("na_color", str))                  
    {
        debug("Set nacolor", data);
        na_color = get_color(data);
    } 
    
    if (!stricmp("button_color", str))             
    {
        debug("Set buttoncolor", data);
        button_color = get_color(data);
    } 
    
    if (!stricmp("outline_color", str))                 
    {
        debug("Set outlinecolor", data);
        outline_color = get_color(data);
    }         
    
    
    if (!stricmp("mysql_host", str))                
    {   
        debug("Set mysql_host", data);     
        strcpy(mysql_host, data);
    }     
    
    if (!stricmp("mysql_user", str))                
    {   
        debug("Set mysql_user", data);     
        strcpy(mysql_user, data);
    }     
    if (!stricmp("mysql_pass", str))                
    {   
        debug("Set mysql_pass", data);     
        strcpy(mysql_pass, data);
    }     
    if (!stricmp("mysql_dbase", str))                
    {   
        debug("Set mysql_dbase", data);     
        strcpy(mysql_dbase, data);
    }     
    
}
/* End of parse_line */


/* loads configuration values from config file */
void config_load(char *config_filename)
{
    FILE *f = fopen(config_filename, "rt");
    
    if (!f)
    {
        debug("config_load", "load_config: Unable to load config file");
        return;
    }    

    debug("Loading lconfig file:", config_filename);

    // load each line
    while (!feof(f))
    {
        char line[STR];    // temperary line
 
        fgets(line, STR, f);

        // check for blank line
        if (line && line[0] > 0)
            parse_line(line);
    }    
    
    fclose(f);
}
/* end of load config */



