/* 
    Automated Drink Mixer

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

    (C) 2007 Jason Hunt
    nulluser@gmail.com
    
    File: database.cpp
*/

#include "core.h"
#include <mysql.h> 

drink_type *drinks;              // Structure of loaded drinks
unsigned int num_drinks = 0;     // Number of loaded drinks

channel_type *channels;                   // Contains the ing_id in the channel
unsigned int num_channels = 0;   // Number of loaded channels
        
static MYSQL *mysql;             // Mysql interface


/* Copy sql error to log file */
void database_error(char *msg)
{
    debug("sql_error", msg);

    char *e = (char *)mysql_error(mysql);

    if (e != NULL)
        debug("sql_error", e);
}


/* Copy sql error to log file */
void database_log(char *data)
{
    char query[STR];
       
    sprintf(query, "insert into log (data, time) values ('%s', NOW())", data);

    if (mysql_query(mysql, query))
    {
        database_error ("Unable to get chanel level\n");        
        return;
    }   
}


/* Copy sql error to log file */
void database_inc_drink(unsigned int drink_num)
{

    unsigned int drink_id = drinks[drink_num].id;
    
    char query[STR];
    
    sprintf(query, "update drinks set count=count+1 where drink_id = '%d'", drink_id);

    if (mysql_query(mysql, query))
    {
        database_error ("Unable to inc drinks count\n");        
        return;
    } 


    // Get current count from database 
    sprintf(query, "select count from drinks where drink_id = '%d'", drink_id);

    if (mysql_query(mysql, query))
    {
        database_error ("Unable to get drink count\n");        
        return;
    } 
    
    MYSQL_RES *result = mysql_store_result(mysql);

    if (result == NULL) return;
   
    MYSQL_ROW row = mysql_fetch_row(result);
    
    if (row == NULL) return;

    drinks[drink_num].count = atoi(row[0]);
}




/* End of sql_error */
void database_sync_channels( void )
{
    // Load all current levels
      
    if (mysql_query(mysql, "select chan_id, level from channels"))
    {
        database_error ("Unable to get chanel level\n");        
        return;
    }   

    MYSQL_RES *result = mysql_store_result(mysql);

    if (result == NULL) return;
   
    MYSQL_ROW row;

    while (row = mysql_fetch_row(result))
    {
        // Number of mixers for this drink is the number of rows in result
            
       if( row[0] == NULL)
       {        
            debug("database_consume_channel", "Unable to get channel level. NULL result");
            return;        
       }    

       if( row[1] == NULL)
       {        
            debug("database_consume_channel", "Unable to get channel level. NULL result");
            return;        
       }    

        unsigned int chan = atoi(row[0]) - 1;

        // Set current level
        channels[chan].level = atof(row[1]);

    }

    mysql_free_result(result);  


   for (int i = 0; i < num_channels; i++)
   {
      if (channels[i].consumed > 0)
      {
          // Subtract amount used
          channels[i].level -= channels[i].consumed;
        
          channels[i].consumed = 0; // Clear consumed

          sprintf(tmp, "update channels set level = '%f' where chan_id = '%d'", 
                  channels[i].level, i+1);

          if (mysql_query(mysql, tmp))
          {
              database_error ("Unable to set channel level\n");        
              return;
          }  
       }
    }    
    
}
/* End of load channels */



/* Load channel configuration from database */
void database_load_channels( void )
{
    num_channels = 0;

    debug("load_channels", "Loading channels");

    if (mysql_query(mysql, "select ing_id, level from channels order by chan_num"))
    {
        database_error ("Unable get mixing channels\n");        
        return;
    }
  
    MYSQL_RES *result = mysql_store_result(mysql);

    if (result == NULL) return;
   
    MYSQL_ROW row;
    
    // Number of mixers for this drink is the number of rows in result
    unsigned int num_load = mysql_num_rows(result);
    
    snprintf(tmp, STR, "Loading %d channel", num_load);
    debug("load_channels", tmp);
    
    
    channels = (channel_type *) malloc(num_load * sizeof(channel_type));
    
    
    while ((row = mysql_fetch_row(result)) && num_channels < num_load)
    {
        if (mysql_num_fields(result) != 2) break;
    
        // Store the ingredient id in the channel
        channels[num_channels].ing_id = atoi(row[0]);
        channels[num_channels].level = atof(row[1]);    
        channels[num_channels].consumed = 0;
        
        
        num_channels++;
        
    }   
          
    mysql_free_result(result);  
}
/* End of load channels */


/* Figure out which channel an ingrdient is on */
bool database_get_channel(unsigned int ing_id,  int &chan_num)
{
    chan_num = NO_CHANNEL; // Indicator for not found
    
    for (unsigned int i = 0; i < num_channels; i++)
    {    
        if (channels[i].ing_id == ing_id)
        {
            chan_num = i+1; // Channels are numbered 1..NUM_CHANNELS
            return(false);
        }
    }
    
    return(true);
}
/* End of get_channel_number */


/* Load mixer data for a specific drink */
void database_load_mixers(unsigned int drink_num, unsigned int drink_id)
{
    drinks[drink_num].mixers_found = true;

    unsigned int max_query = 2048;
    char query[max_query];
    
    snprintf(query, max_query, "select ingredients.name, mixers.ing_id, mixers.amount, \
                    ingredients.cpo,  ingredients.alcohol_content, ingredients.pour_rate  \
                    from mixers,ingredients \
                    where mixers.ing_id = ingredients.ing_id and mixers.drink_id = '%d'", 
                    drink_id);

    if (mysql_query(mysql, query))
    {
        database_error ("Unable get mixers for drink");        
        return;
    }
  
    MYSQL_RES *result = mysql_store_result(mysql);

    if (result == NULL) 
    {
        debug("load_mixers", "No mixers in drink");
        return;
    }
   
    MYSQL_ROW row;
    
    unsigned int loaded_mixers = 0;
    
    // Number of mixers for this drink is the number of rows in result
    unsigned int num_load = mysql_num_rows(result);
    
    snprintf(tmp, STR, "Loading %d mixers", num_load);
    debug("load_mixers", tmp);
    
    // Get memory for mixers
    drinks[drink_num].mixers = (mixer_type *) malloc(num_load * sizeof(mixer_type));
    
    // Load all mixers
    while ((row = mysql_fetch_row(result)) && drinks[drink_num].num_mixers < num_load)
    {
        if (mysql_num_fields(result) != 6) break;
      
        char ing_name[400];
              
        strcpy(ing_name, row[0]);

        unsigned int ing_id = atoi(row[1]);  // ingredient id

        float mix_amount = atof(row[2]);  // Mixer amount   
        float mix_cpo = atof(row[3]);  // Mixer amount   
        float mix_content = atof(row[4]);  // Mixer amount   
        float mix_rate = atof(row[5]);  // Mixer amount   
         
        
        // Get memory for mixer name 
        drinks[drink_num].mixers[loaded_mixers].name = (char *) malloc(strlen(ing_name)+1);
               
        // Copy ingredient name into mixer name
        strcpy(drinks[drink_num].mixers[loaded_mixers].name, ing_name);
        drinks[drink_num].mixers[loaded_mixers].volume = mix_amount;
        drinks[drink_num].mixers[loaded_mixers].cost = mix_cpo;
        drinks[drink_num].mixers[loaded_mixers].content = mix_content;
        drinks[drink_num].mixers[loaded_mixers].pour_rate = mix_rate;

// Update totase size of mixers
        drinks[drink_num].mixer_size += mix_amount;

        int chan_num = NO_CHANNEL;

        if (database_get_channel(ing_id, chan_num))
        {
            debug("load_mixers", "Unable to get channel number");
            drinks[drink_num].mixers_found = false;
        }

        drinks[drink_num].mixers[loaded_mixers].channel = chan_num;


//         if (chan_num != NO_CHANNEL)
//         {
//             if (drinks[drink_num].mixers[loaded_mixers].volume > channels[chan_num-1].level)  
//                  drinks[drink_num].mixers_found = false;   
//        }

        loaded_mixers++;

        snprintf(tmp, STR, "  %4.2f  %s    (chan: %d)", mix_amount, ing_name, chan_num);
        debug("load_mixers", tmp);
    }   
    
    // scale all mixer amount to size of glass
    for (int i = 0; i < loaded_mixers; i++)
    {
             drinks[drink_num].mixers[i].volume = 
            drinks[drink_num].mixers[i].volume /
            drinks[drink_num].mixer_size *
            drinks[drink_num].size;
    }
        
    
    drinks[drink_num].num_mixers = loaded_mixers;        
              
    mysql_free_result(result);           
        
}
/* End of load mixers */


/* Load all drinks from database */
void database_load_drinks( void )
{
    num_drinks = 0;    
    
    if (mysql_query(mysql, "select drink_id, name, size, count from drinks order by name"))
        database_error ("Unable get drink names");
  
    MYSQL_RES *result = mysql_store_result(mysql);
  
    if (result == NULL) 
    {
        database_error("No drinks in database");
        return;
    }
    
    MYSQL_ROW row;
    
    unsigned int num_load = mysql_num_rows(result);
    
   // char t[400];
    snprintf(tmp, STR,  "Number of drinks in database: %d", num_load);
    debug("load_drinks", tmp);
    
    drinks = (drink_type *) malloc(num_load * sizeof(drink_type));
    
    while ((row = mysql_fetch_row(result)) && num_drinks < num_load)
    {
        int num_fields = mysql_num_fields(result);

        if (num_fields != 4 || row[0] == NULL || row[1] == NULL || row[2] == NULL) break;

        char drink_name[400];
    
        strcpy(drink_name, row[1]);
        
        unsigned int drink_id = atoi(row[0]);
                   
        // Get memory for drink name
        drinks[num_drinks].name = (char *) malloc(strlen(drink_name)+1);
         
        strcpy(drinks[num_drinks].name, drink_name);
        
        drinks[num_drinks].num_mixers = 0;
               
        drinks[num_drinks].size = atof(row[2]);
        
        drinks[num_drinks].mixer_size = 0;
        
        drinks[num_drinks].id = drink_id;

        drinks[num_drinks].count = atoi(row[3]);
               
        database_load_mixers(num_drinks, drink_id);                                    

        snprintf(tmp, STR, "Drink name:  %s  Drink ID: %d all_mixers  %d", 
                     drink_name, drink_id, drinks[num_drinks].mixers_found);
        debug("load_drinks", tmp);
                      
        num_drinks++;
    }
        
    mysql_free_result(result);  
}
/* End of load drinks */


/* Load the drink database from the mysql server */
void database_load( void )
{
    debug("load_database", "Loading data from mysql");


                
    database_load_channels();

    database_load_drinks();
  
  


}
/* End of load_database */




void database_unload( void )
{
  for (unsigned int i = 0; i < num_drinks; i++)
    {
        // Free mixer names
        for (int j = 0; j < drinks[i].num_mixers; j++)
            free(drinks[i].mixers[j].name);   
            
        free(drinks[i].mixers); // free entire mixer structure       
          
        free(drinks[i].name); // free name data            
    }
    
    free(channels);
    free(drinks);    
}


void database_start( void )
{
    mysql = mysql_init(NULL); 

    if(mysql == NULL) 
    {
        database_error("mysql_init failed");
        return;
    }

    if (mysql_real_connect(mysql, mysql_host, mysql_user, mysql_pass,
                           mysql_dbase,  0, NULL, 0) == NULL)
    {                            
        database_error("mysql_real_connect failed");
        database_test();
        return;
    }    
    
    
    database_load();
}


/* Free all memory for drink database */
void database_stop( void )
{
    database_unload();
  
     mysql_close(mysql);
    
    debug("database_stop", "done");        
}
/* End of unload_database */


/* Reload the database */
void database_reload( void )
{
    // Do not reload the database if we are making a drink
    if (mixer_making_drink) return;

    database_unload();   
    database_load();
        
    InvalidateRect(main_hwnd, NULL, 0);
     
    display_set_mode(display_mode);
     
    sel_drink = 0;
    sel_base = 0;
    display_mode = DRINK_LIST;
}
/* End of database_reload */


/* Create a test database */
void database_test( void )
{
    if (num_drinks > 0) return;  

    channels = (channel_type *)malloc(sizeof(channel_type) * 3);

//    channels = (int *)malloc(sizeof(unsigned int) * 3);
    num_channels = 3;
    
    
    drinks = (drink_type *) malloc(sizeof(drink_type));
    num_drinks = 1;
 
    drinks[0].name = (char *)malloc(sizeof(char) * STR);
 
    strcpy(drinks[0].name, "White Russian");
 
    drinks[0].mixers = (mixer_type *)malloc(sizeof(mixer_type) * 3);
    drinks[0].num_mixers = 3;
 
    drinks[0].mixers[0].name = (char *)malloc(sizeof(char) * STR);
    strcpy(drinks[0].mixers[0].name, "Vodka");    
    drinks[0].mixers[0].volume = 2;
    drinks[0].mixers[0].channel = 1;
    drinks[0].mixers[0].cost = 1;
    drinks[0].mixers[0].content = 1;
    drinks[0].mixers[0].pour_rate = 1;
    
    drinks[0].mixers[1].name = (char *)malloc(sizeof(char) * STR);
    strcpy(drinks[0].mixers[1].name, "Kahlua");    
    drinks[0].mixers[1].volume = 3;
    drinks[0].mixers[1].channel = 2;
    drinks[0].mixers[1].cost = 1;
    drinks[0].mixers[1].content = 0.3;
    drinks[0].mixers[1].pour_rate = 1;

    drinks[0].mixers[2].name = (char *)malloc(sizeof(char) * STR);
    strcpy(drinks[0].mixers[2].name, "Cream");    
    drinks[0].mixers[2].volume = 4;
    drinks[0].mixers[2].channel = 3;
    drinks[0].mixers[2].cost = 1;
    drinks[0].mixers[2].content = 0;
    drinks[0].mixers[2].pour_rate = 1;    
}
/* End of database test */

