/* 
    Automated Drink Mixer

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

    (C) 2007 Jason Hunt
    nulluser@gmail.com
    
    File: display.cpp
*/

#include "core.h"

int display_x, display_y;       // Dimensions of display

RECT text_rect;                 // Bounding box for draw text

HDC memdc = NULL;               // Memory device context
HBITMAP h_mem_bitmap = NULL;    // Off0screen bitmap

unsigned int v_div_line;        // Position of dividing line
unsigned int arrow_length;      // Length of nav arrow
unsigned int arrow_width;       // Width of nav arrow

HFONT h_cur_font;               // Main large font for lists

int font_height, font_width;    // Dimensions of average font box

int sel_drink = 0;              // Currently selected drink
int sel_base = 0;               // base index for drink scroll

image_type list_image, info_image; // Background images

unsigned int display_mode;      // Current display mode

// Button configuration
unsigned int button_def[NUM_MODES][NUM_BUTTONS] = 
    {{BUTTON_PGUP, BUTTON_UP, BUTTON_GO,   BUTTON_DOWN, BUTTON_PGDOWN},
     {BUTTON_BACK, BUTTON_UP, BUTTON_NONE, BUTTON_DOWN, BUTTON_GO}};                       
                            
                            
/* Change display mode */
void display_set_mode(unsigned int mode)
{
    display_mode = mode;
}
/* End of set_display_mode */
                                 

/* Start an outlined area */       
void display_begin_outline(void)
{
    BeginPath(memdc);    
}
/* End of start outline */


/* Done with area to outline, fill it */
void display_end_outline(COLORREF fill, COLORREF outline)
{
    EndPath(memdc);

    HPEN color_pen = CreatePen(PS_SOLID, 2, outline);
    HBRUSH color_brush =  CreateSolidBrush(fill);

    HPEN old_pen = (HPEN) SelectObject(memdc, color_pen);
    HBRUSH old_brush = (HBRUSH) SelectObject(memdc, color_brush);

    StrokeAndFillPath(memdc);;
   
    SelectObject(memdc, old_brush);
    SelectObject(memdc, old_pen);

    DeleteObject(color_brush); 
    DeleteObject(color_pen);              
}
/* End of end_outline */



/* Draw text with an outline */
void display_draw_outline_text(unsigned int x, unsigned int y, char *text,
                       COLORREF fill, COLORREF outline)
{
    // Move bounding rect for text
    text_rect.left = x;
    text_rect.top = y;
   
    SetTextColor(memdc, fill);

  //  display_begin_outline();   
   
    DrawText(memdc, text, strlen(text), &text_rect, DT_NOCLIP);

//    display_end_outline(fill, outline);
}
/* End of  draw_outline_text */
       
                
                                 
/* Display the list */
void display_drink_list( void  )
{    
    // Number of drinks to show
    unsigned int show_size = drink_list_size;
    
    if (drink_list_size > num_drinks)
        show_size = num_drinks;
                          
    unsigned int cur_y = 0;                          
                                                      
    // Show drinks in current view of list 
    for (unsigned int i = 0; i < show_size; i++)
    {
        if (i + sel_base == sel_drink)
            draw_arrow(arrow_length, cur_y + font_height/2, ARROW_RIGHT, menu_color);   
                
        COLORREF text_color;
        
        if ( drinks[i + sel_base].mixers_found)
            text_color = menu_color;
        else
            text_color = na_color;
                    
        snprintf(tmp, STR, "%s", drinks[i+sel_base].name);       
        display_draw_outline_text(font_width, cur_y, tmp, text_color, outline_color);
        
        cur_y += font_height;
    }
}
/* End of show list */


/* Display the drink info screen */
void display_drink_info( void  )
{    
    if (num_drinks == 0) return;

    // Get pointer to current drink
    drink_type *drink = &drinks[sel_drink];
   
    if (!mixer_making_drink)
    {
        snprintf(tmp, STR, "%s (%d)", drink->name, drink->count);
        display_draw_outline_text(1, 1, tmp, menu_color, outline_color);
    }
        
    unsigned int cur_y = font_height;        
                              
    float cost = 0;
    float total_oz = 0;
    float total_content = 0;    
            
    for (unsigned int j = 0; j < drink->num_mixers; j++)
    {
        COLORREF text_color;
        
        // Get pointer to current mixer
        mixer_type *mixer = &drink->mixers[j];

        unsigned int channel = mixer->channel;        
        
        // Do not draw while making drink
        if (!mixer_making_drink)
        {
            if (channel == NO_CHANNEL)
                text_color = na_color;
            else
            {
                // Display in red if channel is empty
//                if (mixer->volume > channels[channel-1].level)
//                    text_color = na_color;
//                else
                    text_color = menu_color;                        
            }

            cost += mixer->volume * mixer->cost;
                
            total_content += mixer->volume *  mixer->content;
                                        
            snprintf(tmp, STR, "%4.2f oz %s", mixer->volume, mixer->name);

            display_draw_outline_text(font_width, cur_y, tmp, text_color, outline_color);
        }

        cur_y += font_height;
    

        if (channel != NO_CHANNEL)      
        {
            
            // Draw graph if we are making a drink and the ingredient has a channel
            if (mixer_making_drink && channel != NO_CHANNEL)
            {
                draw_graph(font_width, cur_y, 
                           (int)(progress_bar_width * display_x),
                            font_height/2, 
                                                    
                           channels[channel-1].complete);
            }        

            cur_y += font_height/2;
        }
    }
    
    
    unsigned char var;
    
//    get_sensor(var);
  //.  snprintf(tmp, STR, "Sensor: %4d", var);    
//    display_draw_outline_text(400, display_y - 2*font_height, tmp, menu_color, outline_color);    
       
    
    // Do not draw while making drink    
    if (!mixer_making_drink)
    {    
    
        snprintf(tmp, STR, "Size: %5.2f", drink->size);    
        display_draw_outline_text(1, display_y - 2*font_height, tmp, menu_color, outline_color);    

        // Figure about 0.5 oz of ethanol in beer
        if (total_content > 0)
            snprintf(tmp, STR, "Cost: %5.2f   Beers: %5.2f", cost, total_content*2);
        else
            snprintf(tmp, STR, "Cost: %5.2f   Non-Alcoholic ", cost, total_content*2);

        display_draw_outline_text(1, display_y - font_height, tmp, menu_color, outline_color);
    }
       
    
}
/* End of show_drink_info */



void show_waiting( void )
{      
    SelectObject(memdc, (HBRUSH) GetStockObject(BLACK_BRUSH));
    SelectObject(memdc, (HPEN) GetStockObject(WHITE_PEN));  

    Rectangle(memdc, font_width,  font_height,
    v_div_line - font_width, display_y - font_height);
    
    sprintf(tmp, "Waiting for Cup...");
    
    display_draw_outline_text(2*font_width, font_height, tmp, menu_color, outline_color);    

    drink_type *drink = &drinks[sel_drink];

    unsigned int cur_y = 60 + 2 * font_height; 

    bool missing_drink = false;

          
    for (unsigned int j = 0; j < drinks[sel_drink].num_mixers; j++)
    {
        COLORREF text_color;
        
        // Get pointer to current mixer
        mixer_type *mixer = &drink->mixers[j];

        unsigned int channel = mixer->channel;        
        text_color = menu_color;
                  
        // Do not draw while making drink

        if (channel == NO_CHANNEL)
        {                         
            missing_drink = true;            
            snprintf(tmp, STR, "%4.2f oz %s", mixer->volume, mixer->name);

            display_draw_outline_text(3 *font_width, cur_y, tmp, text_color, outline_color);

            cur_y += font_height;
        }      
    }
    
    if (missing_drink)
    {
        sprintf(tmp, " You need to add:");
    
        display_draw_outline_text(2*font_width, 2*font_height, tmp, menu_color, outline_color);    
    }
    
}
 
    
    
    


/* Draw all buttons */
void display_draw_buttons( void )
{
    SelectObject(memdc, (HBRUSH) GetStockObject(BLACK_BRUSH));
    SelectObject(memdc, (HPEN) GetStockObject(WHITE_PEN));    
    
    // Clip over the button area
    Rectangle(memdc, v_div_line, 0, display_x, display_y);
 
    unsigned int but_height = display_y / NUM_BUTTONS;
  
    // Display dividing lines
    for (unsigned int i = 0; i < NUM_BUTTONS-1; i++)
       Rectangle(memdc,v_div_line, (i+1) * but_height, display_x, (i+1) * but_height +2);
   
    // Display buttons
    for (unsigned int i = 0; i < NUM_BUTTONS; i++)
    {
        unsigned int x_pos = (int)(v_div_line + (display_x - v_div_line) / 2.0);
        unsigned int y_pos = (int)(i * but_height + (display_y / 5.0) / 2.0);        
                                           
        unsigned int cur_button = button_def[display_mode][i];                               
                    
        // Override button type with halt when making drink                                                                                             
        if (i == 2 && (mixer_making_drink || mixer_waiting_for_cup) && display_mode == DRINK_INFO)
            cur_button = BUTTON_HALT;


        // Only         display halt when waiting for drink
        if ( !mixer_waiting_for_cup || i==2 )

        switch(cur_button)
        {
            case BUTTON_UP:
                draw_arrow(x_pos, y_pos, ARROW_UP, button_color); break;

            case BUTTON_PGUP:
                draw_arrow(x_pos, y_pos-arrow_length/2, 0, button_color); 
                draw_arrow(x_pos, y_pos+arrow_length/2, 0, button_color);   break;

            case BUTTON_DOWN:
                draw_arrow(x_pos, y_pos, ARROW_DOWN,  button_color); break;

            case BUTTON_PGDOWN:
                draw_arrow(x_pos, y_pos-arrow_length/2, 1, button_color); 
                draw_arrow(x_pos, y_pos+arrow_length/2, 1, button_color);  break;                 

            case BUTTON_BACK:
                draw_arrow(x_pos-arrow_length/2, y_pos, ARROW_LEFT, button_color); break;

            case BUTTON_GO:
                draw_arrow(x_pos+arrow_length/2, y_pos, ARROW_RIGHT, button_color);  break;

            case BUTTON_HALT:
                draw_arrow(x_pos, y_pos, ARROW_LEFT, button_color); 
                draw_arrow(x_pos, y_pos, ARROW_RIGHT, button_color);  break;
        }
    }
}
/* End of draw buttons */


/* Draw entire screen */
void display_draw( void)
{
    // Do not update background while making drink
    if (!mixer_making_drink)
    {
        if (display_mode == DRINK_LIST && list_image.handle != NULL)
             dispaly_show_image(list_image); else

        if (display_mode == DRINK_INFO && info_image.handle != NULL)
             dispaly_show_image(info_image); else

        {   // Just draw a rectangle
            SelectObject(memdc, (HBRUSH) GetStockObject(BLACK_BRUSH));
            SelectObject(memdc, (HPEN) GetStockObject(WHITE_PEN)); 
            Rectangle(memdc, 0, 0,  v_div_line, display_y);
        }
    }
    
    if (display_mode == DRINK_LIST)  display_drink_list();
    if (display_mode == DRINK_INFO)      
    {
        display_drink_info();    

        if (mixer_waiting_for_cup)
            show_waiting();            
    }

    display_draw_buttons();   
}
/* End of draw_screen */


/* Repaint screen from offscreen buffer */
void display_paint(HWND hwnd)
{           
    display_draw();
          
    PAINTSTRUCT ps;
    HDC  hdc = BeginPaint(hwnd, &ps);
    BitBlt(hdc, 0, 0, display_x, display_y, memdc, 0, 0, SRCCOPY);
    EndPaint(hwnd, &ps);  
}
/* End of do_paint */


/* Create large font */
void display_setup_font( HWND hwnd, HDC hdc )
{
    font_height = display_y / drink_list_size;
    font_width = v_div_line / chars_per_line;
    
    h_cur_font = CreateFont(font_height, font_width, 
                            0, 0, FW_BOLD, 0, 0, 0,  
                            ANSI_CHARSET, 
                            OUT_DEFAULT_PRECIS, 
                            CLIP_DEFAULT_PRECIS,
                            ANTIALIASED_QUALITY,
                            DEFAULT_PITCH,
                            "Courier New");

    if (h_cur_font == NULL)
    {
        debug("setup_font", "Unable to create font");
        return;
    }

    SelectObject(memdc, h_cur_font);   
}
/* End of create font */


/* Create off screen buffer */
void display_setup( HWND hwnd )
{
    // Compute arrow size
    arrow_length = (int)(display_x * arrow_length_ratio); 
    arrow_width = (int)(display_x * arrow_width_ratio); 

    // Scale the diviving line from text area to buttons
    v_div_line = display_x - arrow_length * 4;
    
    HDC hdc = GetDC(hwnd);
    memdc = CreateCompatibleDC(hdc);	// Get DC for back buffer

    // Create back buffer
  	h_mem_bitmap = CreateCompatibleBitmap(hdc, display_x, display_y);

  	// Select the bitmap into the back buffer DC
   	SelectObject(memdc, h_mem_bitmap);		

    ReleaseDC(hwnd, hdc);  

    display_setup_font(hwnd, hdc);
    
    GetClientRect(main_hwnd, &text_rect);    // Get clipping rect for text display
    SetBkMode(memdc, TRANSPARENT);           // Transparent background for text
}
/* End of setup_display */


/* Display background image */
void dispaly_show_image(image_type &image)
{
    if (image.handle == NULL) return;

    HDC tmp_dc = CreateCompatibleDC(memdc);
    SelectObject(tmp_dc, image.handle);
    
    StretchBlt(memdc, 0, 0, v_div_line, display_y,     
               tmp_dc, 0, 0, image.x_size, image.y_size, 
               SRCCOPY);

    DeleteDC(tmp_dc);        
}
/* End of display show image */


/* Load image */
void display_load_image(image_type &image)
{ 
    image.handle = (HBITMAP) LoadImage(0, image.filename, IMAGE_BITMAP, 0, 0, LR_LOADFROMFILE);
    
    if (image.handle == NULL)
    {
        debug("display_start", "Unable to load bitmap");
        return;
    } 
    
    BITMAP bm;
    GetObject(image.handle, sizeof(bm), &bm);

    // Remember the size of the bitmap
    image.x_size = bm.bmWidth;
    image.y_size = bm.bmHeight;   
}
/* End of display_load_image */



/* Init the display */
void display_start( void )
{
    // Disable mouse curser in touch screen mode
    if (touchscreen_mode) ShowCursor(false);
 
    display_setup(main_hwnd);    
 
    display_load_image(list_image);
    display_load_image(info_image);
}
/* End of start_display */



/* Resize the display and update buffers */
void display_resize(HWND hwnd, LPARAM l)
{
    display_x = LOWORD(l);
    display_y = HIWORD(l);

    DeleteObject(h_mem_bitmap);		//	 Delete old screen buffer
    DeleteDC(memdc);		      	// Delete old memory DC

    display_setup(hwnd);

    InvalidateRect(hwnd, NULL, true);   
}
/* End of resize_display */


/* Free display variables */
void display_stop( void )
{
    DeleteObject(h_mem_bitmap);		//	 Delete off screen buffer

    DeleteObject(list_image.handle);		//	 Delete off screen buffer
    DeleteObject(info_image.handle);		//	 Delete off screen buffer

    DeleteDC(memdc);	       		// Delete memory DC
    
    // Re-enable mouse cursor
    if (touchscreen_mode)
        ShowCursor(true);  

    debug("display_stop", "done");
}
/* End of shutdown_display */


     
        
