/* 
    Automated Drink Mixer

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

    (C) 2007 Jason Hunt
    nulluser@gmail.com
    
    File: main.cpp
*/

#include "core.h"

HWND main_hwnd;         // Handle to main window

// Number of seconds program has been inacive
unsigned int inactive_counter = 0; 


/* Called before window is created */
void system_init( void )        
{
    srand(time(0));

    sound_start();               // Must be head because the config can load sounds
    config_defaults();           // Set default values

    config_load("config.txt");
}
/* End of system_init */


/* Called during main window creation */
void system_start(HWND hwnd)
{             
    main_hwnd = hwnd;

    // Start the modules
    display_start();    
    database_start();
  
  start_serial();
    
    // Display drink list    
    display_set_mode(DRINK_LIST);    

    SetTimer(hwnd, SECOND_TIMER, 1000, NULL);    
    SetTimer(hwnd, CHECK_TIMER, 100, NULL);
    
}
/* End of system start */
    

/* Called to shutdown program */    
void system_shutdown( void )
{
    mixer_close_all();

    database_stop();
    display_stop();  
    sound_stop();
          
    stop_serial();
          
    PostQuitMessage(0);  
    
    debug("shutdown", "done");              
}
/* End of system_shutdown */    


/* Deal with timer messages */
void timer_message(WPARAM w, LPARAM l)
{
    // Is this timer for a channel ?
    if (w == SECOND_TIMER)
    {

        if (mixer_waiting_for_cup)
            inactive_counter = 0;

        inactive_counter++;
                  
        // Inactive for a while, reload database
        if (inactive_counter > inactive_timeout)
        {
            database_reload();       
            inactive_counter = 0;       
        }
    }            


   if (w == CHECK_TIMER)
   { 
        check_cup();
//          InvalidateRect(main_hwnd, NULL, 0);            
        
    }


    // Message for channel close
    if (w >= CHANNEL_TIMER&& w < UPDATE_TIMER)
        mixer_close_channel(w - CHANNEL_TIMER);

    // Message for channel update
    if (w == UPDATE_TIMER)    
        mixer_update_channels();
}
/* End of timer_message */


/* Deal with keypress */
void keydown_message(WPARAM w, LPARAM l)
{
    inactive_counter = 0;       
        
    if (w == 'R') database_reload();                            
//    if (w == 'T') database_test();

    if (w == VK_SPACE) mixer_cup_override = true;


    if (w == 27) DestroyWindow(main_hwnd);   // Escape exits        
        
    if (display_mode == DRINK_LIST)
    {
        if (w == VK_RETURN) display_set_mode(DRINK_INFO);

        if (w == VK_UP) input_list_up();
        if (w == VK_DOWN) input_list_down();
           
    }else
    if (display_mode == DRINK_INFO)
    {
        if (w == VK_BACK) display_set_mode(DRINK_LIST);
        if (w == VK_RETURN) mixer_make_cur_drink();  
    }
        
    InvalidateRect(main_hwnd, NULL, 0);
}
/* End of keydown_message */


/* Deal with mouse up */
void mouseup_message(WPARAM w, LPARAM l)
{
    inactive_counter = 0;       // No longer inactive

    unsigned int x = LOWORD(l);
    unsigned int y = HIWORD(l);
                          
    if (x > v_div_line-150) // Mouse in the button area?
    {
        input_button_press(x, y);
    } else
    {
        // Pointer drink selection                
        if (display_mode == DRINK_LIST)
            input_select_drink(y / (display_y / drink_list_size));
    }
        
    InvalidateRect(main_hwnd, NULL, 0);            
}
/* End of mouseup message */


/* The window function */
LRESULT CALLBACK WinProc(HWND hwnd, UINT msg, WPARAM w, LPARAM l)
{
    switch(msg)
    {
        case WM_TIMER:     timer_message(w, l);     return(0);
        case WM_CREATE:    system_start(hwnd);      return(0);
        case WM_KEYDOWN:   keydown_message(w, l);   return(0);
        case WM_PAINT:     display_paint(hwnd);     return(0);
        case WM_LBUTTONUP: mouseup_message(w, l);   return(0);  
        case WM_DESTROY:   system_shutdown();       return(0);
        case WM_SIZE:      display_resize(hwnd, l); return(0);

        default: return(DefWindowProc(hwnd, msg, w, l));
    }

    return(0);  
}
/* End of WinProc */


/* WinMain */
int WINAPI WinMain(HINSTANCE h_this_inst, HINSTANCE h_prev_inst, LPSTR args, int mode)
{
    system_init();

    WNDCLASS wnd_class;

    char program_name[] = "DOM";
        
    wnd_class.hInstance = h_this_inst;
    wnd_class.lpszClassName = program_name;
    wnd_class.lpfnWndProc = WinProc;
    wnd_class.style = CS_OWNDC;
    
    wnd_class.hIcon = LoadIcon(NULL, IDI_APPLICATION);
    wnd_class.hCursor = LoadCursor(NULL, IDC_ARROW);
    wnd_class.lpszMenuName = NULL;
    
    wnd_class.cbClsExtra = 0;
    wnd_class.cbWndExtra = 0;
    
    wnd_class.hbrBackground = NULL; // Not needed
    
    if (!RegisterClass(&wnd_class)) return(0);
                
    unsigned int window_options;
    
    // Setup the window mode
    if (fullscreen)
        window_options = WS_POPUP | WS_VISIBLE;
    else
        window_options = WS_OVERLAPPEDWINDOW;
        
    // Create the window
    main_hwnd = CreateWindow(program_name, "Drink-O-Matic", 
                             window_options,
                             0, 0,                     // Pos
                             display_x, display_y,     // Size
                             HWND_DESKTOP, NULL, h_this_inst, NULL);

    ShowWindow(main_hwnd, mode);
    UpdateWindow(main_hwnd);
           
    MSG msg;
    
    // Windows message loop
    while(GetMessage(&msg, NULL, 0, 0))
    { 
        TranslateMessage(&msg);
        DispatchMessage(&msg);                            
    }
                       
    debug("winmain", "returning");                                     
                        
    return(msg.wParam);
}
/* End of winmain */



