/*
    Robot Interface
    (C) 2006 Jason Hunt
    nulluser@gmail.com
    
    File: serial.cpp
*/


//#include <windows.h>
//#include <stdio.h>

#include "core.h"

//#include "serial.h"
//#include "robot.h"
//#include "display.h"


HANDLE h_serial = NULL;			      // Handle for serial port

bool serial_ok = false;               // True if serial is working 

HANDLE h_serial_thread = NULL;               // Handel for serial control thread
DWORD serial_thread_id;               // Serial control 
bool serial_running = false;          // True if serial thread is running


bool     got_serial_data = false;


CRITICAL_SECTION serial_thread_lock;  // Lock for serial thread

bool timeout_flag = false;            // Set once for a timeout condition

//char serial_port[] = "COM1";                 // Filename of serial port

unsigned char sensor_data = 0;



/* Make sure there is no pending data for the next retry */
void flush_serial( void )
{
    DWORD read = 1;
                    
    // Wait for byte
    while (read)
    {
        char ch;
        ReadFile (h_serial, &ch, 1, &read, NULL);  
    }  
} 
/* End of flush serial */



void write_serial(char ch)
{
    
    char buffer[2];
    
    buffer[0] = ch;
    
    DWORD len = 1;
    	// Send data to controller
   	if (!WriteFile(h_serial, buffer, 1, &len, NULL))
  	{
       debug("write_serial", "Unable to send data");
        serial_ok = false;
        //return(1);
    }   
    
  //  debug("write", buffer);
    
    
}



bool get_sensor( unsigned char &data )
{
    if (!serial_ok) return(false);

    unsigned char val = 0;

    bool ret = true;

    EnterCriticalSection(&serial_thread_lock);
    data = sensor_data;
    
    if (got_serial_data == false)
        ret = false;
    
    LeaveCriticalSection(&serial_thread_lock);   
    
    return(ret);
}



/* Core thread for serial transfer */
DWORD WINAPI serial_thread(LPVOID p)
{
    DWORD length;

    bool done = false;

    unsigned char buffer[BUFFER_SIZE];



    while (!done)
    {

       // Read telemetry data from the controller 
        ReadFile(h_serial, buffer, 1, &length, NULL);
    
        if (length > 0)
        {
            EnterCriticalSection(&serial_thread_lock);

            got_serial_data  = true;

            done = !serial_running;
            if (!serial_ok) done = true;
            sensor_data = buffer[length-1];
        
            LeaveCriticalSection(&serial_thread_lock);
         }
         
         Sleep(10);
   }
        
    return(0);
}
/* End of serial thread */



/* Starts the serial system */
void start_serial( void )
{
    InitializeCriticalSection(&serial_thread_lock);    
    
	serial_ok = false;
  
	h_serial = CreateFile(serial_port,							// Port name 
			 			  GENERIC_READ | GENERIC_WRITE,		// File access
		                  0,								// share mode
			 			  NULL,								// Security
			 			  OPEN_ALWAYS,						// Creation					
			 			  0,								// Flags
						  NULL );							// Template

	if (h_serial == INVALID_HANDLE_VALUE)
	{
        char b[400];
        sprintf(b, "Unable to open serial port (%s)", serial_port);
        debug("serial_start", b);

		return;
	}

	DCB serial_param;

	if (!GetCommState(h_serial, &serial_param))
	{
		debug("serial_start", "Unable to get commstate");
		return;
	}

    serial_param.BaudRate = CBR_19200;
	serial_param.fParity = false;	
    serial_param.fBinary  = true;
	serial_param.fOutxCtsFlow = false;
	serial_param.fOutxDsrFlow = false;
	serial_param.fDtrControl = DTR_CONTROL_DISABLE;
    serial_param.fDsrSensitivity = false;
    serial_param.fErrorChar = false;
    serial_param.fNull = false;
	serial_param.fOutX = false;
	serial_param.fInX = false;
    serial_param.fRtsControl = RTS_CONTROL_DISABLE;		// Disable request to send control
    serial_param.Parity = NOPARITY;
    serial_param.ByteSize = 8;
	serial_param.StopBits = TWOSTOPBITS;

	if (!SetCommState(h_serial, &serial_param))
	{
		debug("serial_start", "Unable to set commstate");
		return;
	}

	COMMTIMEOUTS timeouts;

	timeouts.ReadIntervalTimeout = 200;
	
	timeouts.ReadTotalTimeoutMultiplier = 1;
	timeouts.ReadTotalTimeoutConstant = 200; 
	timeouts.WriteTotalTimeoutMultiplier = 1;
	timeouts.WriteTotalTimeoutConstant = 200;

	if (!SetCommTimeouts(h_serial, &timeouts))
	{
		debug("Serial_start", "Unable to get comm timeouts");
		return;
	}


    // Set serial thread active
    serial_running = true;

    // Start the serial therad
    h_serial_thread = CreateThread(0, 0, serial_thread, NULL, 0, &serial_thread_id);

	serial_ok = true;
	
	debug("serial_Start", "Serial Online");
}
/* End of start serial */



/* Close the serial port */
void stop_serial( void )
{    
    if (h_serial_thread != NULL)
    {
        EnterCriticalSection(&serial_thread_lock);
        serial_running = false;                         // Tell thread to exit
        LeaveCriticalSection(&serial_thread_lock);

        WaitForSingleObject(h_serial_thread, 1000); // Wait for thread to terminate
    }

    if (h_serial != NULL)
    	CloseHandle(h_serial);
}
/* End of stop serial */



