/* 
    Automated Drink Mixer

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

    (C) 2007 Jason Hunt
    nulluser@gmail.com
    
    File: sound.cpp
*/


#include "core.h"

// Linked list type for sound paths
typedef struct sound_path_type
{
    char *path;
    sound_path_type *next;
};


sound_path_type *sound_paths = NULL;   // All sound path data
unsigned int num_sounds = 0;           // Number of loaded sound paths


/* Add sound filename to linked list */
void sound_add_file(char *root_path, char *filename)
{    
    // Ignore reletive paths
    if (!strcmp(filename, ".")) return;
    if (!strcmp(filename, "..")) return;

    // Compute length of path and account for the added "\"
    unsigned int len = strlen(root_path) + strlen(filename) + 2;

    char *tmp_path_str = (char *) malloc(sizeof(char) * len);
    
    strcpy(tmp_path_str, root_path);
    strcat(tmp_path_str, "\\");
    strcat(tmp_path_str, filename);
    
    debug("Found sound: ", tmp_path_str);
    
    sound_path_type *tmp_path = (sound_path_type*)malloc(sizeof(sound_path_type));
    
    tmp_path->next = sound_paths;
    tmp_path->path = tmp_path_str;
    
    sound_paths = tmp_path;
    
    num_sounds++;
}
/* End of sound_add_file */


/* Load sound filenames */
void sound_load_path( char *root_path )
{
    debug("Loading sounds from", root_path);
    
    char search_path[500];
    strcpy(search_path, root_path);
    strcat(search_path, "\\*");
        
    WIN32_FIND_DATA search;
    
    HANDLE h_search = FindFirstFile(search_path, &search);
        
    if (h_search == INVALID_HANDLE_VALUE)
    {
        debug("sound_load", "Unable to load sounds");
        return;
    }
    // Add first found file
    sound_add_file(root_path, search.cFileName);

    // Keep searching for files
    while (FindNextFile(h_search, &search))
        sound_add_file(root_path, search.cFileName);
    
    FindClose(h_search);    
}
/* End of sound_load_path */


/* Init sound system */
void sound_start( void )
{
    num_sounds = 0;   
}
/* End of sound_start */


/* Unload sound path data */
void sound_stop( void )
{
    while (sound_paths)
    {
        sound_path_type *tmp = sound_paths->next;
    
        free(sound_paths->path);
        free(sound_paths);
    
        sound_paths = tmp;
    }
    
    debug("sound_stop", "done");    
}
/* End of sound_unload */


/* Play a random sound */
void sound_play_random( void )
{
    if (num_sounds == 0) return;

    snprintf(tmp, STR, "Num sounds = %d", num_sounds);
    debug("sounds", tmp);
    
    sound_path_type *tmp_path = sound_paths;

    unsigned int sound_num = rand() % num_sounds;

    snprintf(tmp, STR, "Picked %d",sound_num);
    debug("sounds", tmp);

    while (tmp_path && sound_num)
    {
        tmp_path = tmp_path->next;
        sound_num--;
    }

    if (tmp_path)
    {
        PlaySound(tmp_path->path, 0, SND_FILENAME | SND_ASYNC);
       debug("sound play", tmp_path->path);
    }
    
}
/* End of sound_play_random */



