/* 
    Automated Drink Mixer

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

    (C) 2007 Jason Hunt
    nulluser@gmail.com
    
    File: types.cpp
*/


#ifndef TYPES_H
#define TYPES_H

#define STR 1024

// Mysql database connection info
#define MYSQL_HOST   "127.0.0.1"
#define MYSQL_USER   "localbar"
#define MYSQL_PASS   "localbar"
#define MYSQL_DBASE  "bar"

// Display modes
#define DRINK_LIST  0
#define DRINK_INFO  1

// Hard coded constants
#define NUM_MODES                  2  // Number of display modes
#define NUM_BUTTONS                5  // Number of command buttons

#define CHANNEL_TIMER           1024  // Base for channel timers
#define UPDATE_TIMER            2048  // Timer ID for update


// Different types of command buttons
#define BUTTON_NONE     0
#define BUTTON_UP       1
#define BUTTON_PGUP     2
#define BUTTON_DOWN     3
#define BUTTON_PGDOWN   4
#define BUTTON_BACK     5
#define BUTTON_GO       6
#define BUTTON_HALT     7

// Arrow defs
#define ARROW_UP        0
#define ARROW_DOWN      1
#define ARROW_LEFT      2
#define ARROW_RIGHT     3

// Channel not found indicator
#define NO_CHANNEL     -1

// Background image type
typedef struct 
{
    unsigned int x_size, y_size;
    char filename[STR];
    HBITMAP handle; 
} image_type;


// Stats for channel
typedef struct 
{
    float start_time;       // Time channel was opened
    float duration;         // How long the channel needs to be on
    float complete;         // Completation percentage
    float level;            // Current level
    float consumed;
    unsigned int ing_id;
           
} channel_type;


// Data for a mixer
typedef struct
{
    char *name;                 // Ingredient name
    int channel;                // Channel the ingredient is loaded into
    float volume;               // Amount of pour
    float cost;                 // Cost per ounce
    float content;              // Alchol content
    float pour_rate;            // Pour rate of drink
} mixer_type;


// Data for a drink
typedef struct
{
    char *name;                 // Name of drink
    mixer_type *mixers;         // Structure of mixers
    unsigned int num_mixers;    // Number of mixers
    bool mixers_found;          // True if all mixers are avaiable
    float size;                 // Size in OZ to scale drink to
    float mixer_size;           // Total size of mixers for scaling
    unsigned int id;
    unsigned int count;
} drink_type;

#endif

