/* 
    Automated Drink Mixer

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

    (C) 2007 Jason Hunt
    nulluser@gmail.com
    
    File: utility.cpp
*/

#include "core.h"

char tmp[STR];  //Global tmp char buffer

/* Draw a colored rectangle */
void color_rect(int x, int y, 
                int x_size, int y_size, 
                COLORREF color)
{
    HPEN color_pen = CreatePen(PS_SOLID, 1, color);
    HBRUSH color_brush =  CreateSolidBrush(color);

    HPEN old_pen = (HPEN) SelectObject(memdc, color_pen);
    HBRUSH old_brush = (HBRUSH) SelectObject(memdc, color_brush);

    Rectangle(memdc, x, y, x+x_size, y+y_size);

    SelectObject(memdc, old_brush);
    SelectObject(memdc, old_pen);

    DeleteObject(color_brush); 
    DeleteObject(color_pen);    
}
/* End of colorrect */


/* Draw a bar graph */
void draw_graph(int x, int y, int x_size, int y_size, float value)
{
    int cut = (int)(x_size * value);

    color_rect(x, y, cut, y_size, RGB(0,0,255));

    color_rect(x+cut, y, x_size-cut, y_size, RGB(255,0,0));
}
/* End of draw_graph */



/* Display an arrow */
void draw_arrow(unsigned int x, unsigned int y, unsigned int dir, 
                COLORREF color)
{
    HPEN wide_pen = CreatePen(PS_SOLID, arrow_width, color);

    HPEN old_pen =(HPEN) SelectObject(memdc, wide_pen);
        
    if (dir == ARROW_UP) // up
    {
    
        MoveToEx(memdc, x-arrow_length, y+arrow_length, 0);    
        LineTo(memdc, x, y);    
        LineTo(memdc, x+arrow_length, y+arrow_length);    
    }
    else
    if (dir == ARROW_DOWN) // down
    {
    
        MoveToEx(memdc, x-arrow_length, y-arrow_length, 0);    
        LineTo(memdc, x, y);    
        LineTo(memdc, x+arrow_length, y-arrow_length);    
    }
    if (dir == ARROW_LEFT) // left
    {
    
        MoveToEx(memdc, x+arrow_length, y-arrow_length, 0);    
        LineTo(memdc, x, y);    
        LineTo(memdc, x+arrow_length, y+arrow_length);    
    }
    if (dir == ARROW_RIGHT) // right
    {
    
        MoveToEx(memdc, x-arrow_length, y-arrow_length, 0);    
        LineTo(memdc, x, y);    
        LineTo(memdc, x-arrow_length, y+arrow_length);    
    }
    

    SelectObject(memdc, old_pen);

    DeleteObject(wide_pen);
}
/* End of display arrow */



/* Return current time in seconds */
double get_time( void )
{
  //  EnterCriticalSection(&time_lock);           

    static LARGE_INTEGER freq;
    bool init = 0;

    if (!init)
    {
        QueryPerformanceFrequency(&freq);
        init = 1;
    }

    LARGE_INTEGER c;

    QueryPerformanceCounter(&c);

    double value = c.QuadPart / (double)freq.QuadPart;
    
   // LeaveCriticalSection(&time_lock);
    
    return(value);
}
/* End of get_time */
