
/*
    Robot Controller Source
	(C) 2006 Jason Hunt nulluser@gmail.com

	Version 0.01

	16f871.h has been modified as follows:
	
	// original
	// #pragma chip PIC16F871, core 14, code 2048, ram 32 : 0xBF /0 /3

	#pragma chip PIC16F871, core 14, code 0x1F00, ram 32 : 0x1FF
	#pragma cdata[0] = 0x0180 + 0x0A // CLRF PCLATH, prepare for code page change
	#pragma cdata[1] = 0 // NOP
	#pragma cdata[2] = 0 // NOP
	#pragma resetVector 3 // start address for user program

	This reserves space for the serial bootloader.
*/

#include <int16cxx.h>

#pragma chip PIC16f871
#pragma config |= 0x3FF2 			// For 20 mhz ceramic resinator


#define NUM_RECV_BYTES	11			// Bytes in control packet
#define NUM_SEND_BYTES	20			// Bytes in telemetery packet

#define NUM_SERVOS	6	   		    // Number of servo outputs

#define NUM_ANALOG 8

/* Pin Assignments */

//#pragma bit led_0 @ PORTB.6

#pragma bit channel_01 @ PORTD.3
#pragma bit channel_02 @ PORTC.4
#pragma bit channel_03 @ PORTC.5
#pragma bit channel_04 @ PORTD.4
#pragma bit channel_05 @ PORTD.5
#pragma bit channel_06 @ PORTD.6
#pragma bit channel_07 @ PORTD.2
#pragma bit channel_08 @ PORTD.1
#pragma bit channel_09 @ PORTD.0
#pragma bit channel_10 @ PORTC.3
#pragma bit channel_11 @ PORTC.2
#pragma bit channel_12 @ PORTC.1
#pragma bit channel_13 @ PORTD.7
#pragma bit channel_14 @ PORTC.0



/*******************
** Interrupt Code **
*******************/

#pragma interruptSaveCheck  n
#pragma origin 4


unsigned char analog_data;

unsigned int wait_analog ;

void rx_int( void );

unsigned int t;


/* the ISR to update the servos */
interrupt ISR(void)
{
    int_save_registers

    if (RCIF) rx_int();

	if (TMR2IF)	
	{
		if (wait_analog && !GO)
		{
			// Reduce wait count
			wait_analog--;

			// Done waiting, start the converter
			if (wait_analog == 0) 
      		{
                GO = 1;
				TXREG = analog_data;
			}
		}

		TMR2IF = 0; // clear flag
	}

	// The analog to digital converter is done
    if (ADIF)
    {  
		analog_data = ADRESH & 0x03;
		analog_data *= 64;
		analog_data |= ADRESL;

		TXREG = analog_data;

    	ADCON0 = 0b.1100.0111;

		// Setup to wait to fire the go signal

		wait_analog = 5;

		GO = 0;
		ADIF = 0;		// Clear flag
    }

    int_restore_registers
}
/* end of interrupt service routine */

/**************************
** End of interrupt Code **
**************************/



void rx_int( void )
{
    RCIF = 0;

    char ch = RCREG;  // Normal char for packet

    unsigned char state = 0;

    if (ch & 0x80) state = 1;

    unsigned char channel = ch & 0x7f;

    // Decode channel and set state
    if (channel == 1) channel_01 = state; else
    if (channel == 2) channel_02 = state; else
    if (channel == 3) channel_03 = state; else
    if (channel == 4) channel_04 = state; else
    if (channel == 5) channel_05 = state; else
    if (channel == 6) channel_06 = state; else
    if (channel == 7) channel_07 = state; else
    if (channel == 8) channel_08 = state; else
    if (channel == 9) channel_09 = state; else
    if (channel == 10) channel_10 = state; else
    if (channel == 11) channel_11 = state; else
    if (channel == 12) channel_12 = state; else
    if (channel == 13) channel_13 = state; else
    if (channel == 14) channel_14 = state; 

} 






/***********************
**	Serial functions **
***********************/

/* Set the serial port to  9600 8N1 */
void setup_serial(void)
{
//	SPBRG = 129;   // divisor for 9600 baud
	SPBRG = 64;    // divisor for 19200 baud

    TXSTA = 0b.0010.0100;  // transmitt enable, high speed
	RCSTA = 0b.1001.0000; // serial enable

	char ch = RCREG;  // flush buffer
	ch = RCREG;
	ch = RCREG;
}
/* end of setup serial */




/*******************
** Initialization **
*******************/

/* system startup */
void startup( void )
{
	PORTA = 0;			// clear ports
	PORTB = 0;
	PORTC = 0;
	PORTD = 0;
	PORTE = 0;

	TRISA = 0b.1111.1111;	// all inputs
	TRISB = 0b.0000.0000;	
	TRISC = 0b.1000.0000;	// bit 7 is rx
	TRISD = 0b.0000.0000;	
	TRISE = 0b.0000.0000;	// E0-E2 digital in

	// clear core values before interrupts are turned on

	// setup timer 0
    OPTION = 4;				// prescaler divide by 16
	T0CS = 0;				// internel clock
    TMR0 = 0;				// clear the timer
    T0IE = 1;				// enable interrupt for timer 0

	// setup timer 1
	TMR1L = 0;			// reset timer
	TMR1H = 0;

	TMR1ON = 0; 			// shut off timer to modify values

//T1CON = 0b00110101;
//
	//T1CKPS1 = 1;			// setup prescaler
	//T1CKPS0 = 0;
	//T1OSCEN = 0;			// shut off oscillator

	TMR1CS = 0;				// internal clock
	TMR1IE = 0;				// enable interrupt

	TMR1ON = 1; 			// turn timer on
	TMR1IF = 0;				// clear interrupt flag

//	TMR2 = 0xe0;

    T2CON = 0b01111111;

	//TMR2ON = 1;				// enable timer 2
	TMR2IE = 1;


   RCIF = 0;	// Clear receive interrupt
   RCIE = 1; 	// Enable receive interrupts

   TXIF = 0;	// Clear receive interrupt
   TXIE = 1; 	// Enable receive interrupts

   analog_data = 0;


	ADCON1 = 0b.1000.0000;	// RE0, 1, 2 : digital RA0,1,2,3,5 : analog


	ADCS1 = 1;				// Fosc/32 clocl
	ADCS0 = 0;
	ADCON0 &= 0b.1100.0111;
	ADON = 1;					// turn on converter

    ADIF =0;
    ADIE =1;

	GO = 1;
	
    GIE = 1;				// interrupts allowed 
	PEIE = 1;				// enable periphial interrupts

wait_analog = 1;

    setup_serial();
}

/* end of startup */

/**************************
** End of Initialization **
**************************/





/* main function */
void main ( void )
{
	startup();


	while (1);	// infinte loop
}
/* end of main */
